This archive contains the distribution Rex,
version 1.12.0:

  the friendly automation framework

This software is Copyright (c) 2020 by Jan Gehring.

This is free software, licensed under:

  The Apache License, Version 2.0, January 2004


This README file was generated by Dist::Zilla::Plugin::Readme v6.015.
